functions{
  real nngp_w_lpdf(vector w_r, real sigmasq, real phi, matrix neardist, matrix neardistM,
                   int[,] nearind, int n, int M){
    vector[n] V;
    vector[n] Uw;
    vector[n] w;
    int dim;
    int h;
    Uw = w_r;
    for (i in 2:n) {
      matrix[ i < (M + 1)? (i - 1) : M, i < (M + 1)? (i - 1): M] temp_neardistM;
      matrix[ i < (M + 1)? (i - 1) : M, i < (M + 1)? (i - 1): M] L;
      vector[ i < (M + 1)? (i - 1) : M] u;
      vector[ i < (M + 1)? (i - 1) : M] v;
      row_vector[i < (M + 1)? (i - 1) : M] v2;
      dim = (i < (M + 1))? (i-1) : M;
      if(dim == 1){temp_neardistM[1, 1] = 1;}
      else{
        h = 0;
        for (j in 1:(dim - 1)){
          for (k in (j + 1):dim){
            h = h + 1;
            temp_neardistM[j, k] = exp(- neardistM[(i - 1), h] / phi);
            temp_neardistM[k, j] = temp_neardistM[j, k];
          }
        }
        for(j in 1:dim){
          temp_neardistM[j, j] = 1;
        }
      }
      L = cholesky_decompose(temp_neardistM);
      u = to_vector(exp(- neardist[(i - 1), 1:dim] / phi));
      v = mdivide_left_tri_low(L, u);
      V[i] = 1 - (v' * v);
      v2 = mdivide_right_tri_low(v', L);
      Uw[i] = Uw[i] - v2[1:dim] * w_r[nearind[(i - 1),1:dim]];  
      }
    V[1] = 1;
    return - 0.5 * (sum( log(V)  + 1/ sigmasq * (Uw .* Uw ./ V)) + n * log(sigmasq));
  }
  }

data {
  int n_point;
  int n_area;
  int M;
  int aL;
  vector[n_point] Y;
  int p_point; // number of coefficient for point
  int p_area; // number of coefficient for area
  matrix[n_point, p_point] X_point; // design matrix for point
  matrix[n_area, p_area] X_area; // design matrix for area
  int nearind[(n_point+aL) - 1, M];
  matrix[(n_point+aL) - 1, M] neardist;
  matrix[(n_point+aL) - 1, (M * (M - 1) / 2)] neardistM;
  matrix[n_area, aL] A1; // aggregation matrix for areal
  int Q[n_area]; // response at areas
}

parameters{
  vector[p_point] beta_point; // coefficients
  vector[p_area] beta_area;
  real<lower = 0> sigma_sq;
  real<lower = 0> tau_sq;
  real<lower = 0> phi;
  vector[n_point+aL] w;
}

transformed parameters{
  vector[n_point] A0w;
  vector[n_area] A1w;  
  A0w = w[1:n_point];
  A1w = log(A1 * exp(w[(n_point+1):(n_point+aL)]));
}

model{
  beta_point ~ normal(0, 5);
  beta_area ~ normal(0, 5);
  sigma_sq ~ inv_gamma(2, 1);
  tau_sq ~ inv_gamma(2, 1);
  phi ~ normal(300,100);
  w ~ nngp_w(sigma_sq, phi, neardist, neardistM, nearind, n_point+aL, M);
  Y ~ normal(X_point * beta_point + A0w, sqrt(tau_sq));
  Q ~ poisson_log(X_area * beta_area + A1w);
}
