data {
  int n_point;
  int n_area;
  int M;
  int aL;
  vector[n_point] Y;
  int p_point; // number of coefficient for point
  int p_area; // number of coefficient for area
  matrix[n_point, p_point] X_point; // design matrix for point
  matrix[n_area, p_area] X_area; // design matrix for area
  int nearind[n_point - 1, M];
  vector[M] sC_site_nei[n_point - 1];
  matrix[M, M] sC_nei[n_point - 1];
  int nearind_sample[aL, M];
  matrix[M, M] C_nei[aL];
  vector[M] C_site_nei[aL];
  matrix[n_area, aL] A1; // aggregation matrix for areal
  int Q[n_area]; // response at areas
}

parameters{
  vector[p_point] beta_point;
  vector[p_area] beta_area;
  real<lower = 0> sigma_sq;
  real<lower = 0> tau_sq;
  real<lower = 0> phi;
  vector[n_point + aL] noise;
}

transformed parameters{
  vector[n_point] w;
  vector[n_point] w_var;
  vector[aL] wa; 
  vector[aL] wa_var; 
  vector[n_area] wA;
  vector[M] C_site_nei_C_nei_inv;
  vector[M] C_site_nei_phi;
  matrix[M,M] C_nei_phi;
  w_var[1] = sigma_sq;
  w[1] = sqrt(w_var[1]) * noise[1];
  // for transforming w
   for (i in 2:n_point) {
      int dim;
      matrix[ i < (M + 1)? (i - 1) : M, i < (M + 1)? (i - 1): M] sC_nei_phi;
      vector[ i < (M + 1)? (i - 1) : M] sC_site_nei_phi;
      vector[ i < (M + 1)? (i - 1) : M] sC_site_nei_C_nei_inv;
      dim = (i < (M + 1))? (i-1) : M;
      if(dim == 1){sC_nei_phi[1, 1] = 1;}
      else{
        for (j in 1:dim){
          for (k in j:dim){
            sC_nei_phi[j, k] = exp(sC_nei[(i - 1)][j,k] / phi);
            sC_nei_phi[k, j] = sC_nei_phi[j, k];
          }
        }
      }
      sC_site_nei_phi = exp(sC_site_nei[(i - 1)][1:dim] / phi);
      sC_site_nei_C_nei_inv = mdivide_left_spd(sC_nei_phi, sC_site_nei_phi);
      w_var[i] = sigma_sq * (1 - dot_product(sC_site_nei_C_nei_inv, sC_site_nei_phi));
      w[i] = dot_product(sC_site_nei_C_nei_inv , w[nearind[i-1,1:dim]]) + sqrt(w_var[i]) * noise[i];
    }
  // for transforming wa
  for (i in 1:aL) {
     for (j in 1:M){
       C_site_nei_phi[j] = exp(C_site_nei[i][j]/phi);
     }
     for (k in 1:M){
       for (l in k:M){
         C_nei_phi[k,l] = exp(C_nei[i][k,l]/phi);
         C_nei_phi[l,k] = C_nei_phi[k,l];
       }
     }
     C_site_nei_C_nei_inv = mdivide_left_spd(C_nei_phi, C_site_nei_phi);
     wa_var[i] = sigma_sq * (1 - dot_product(C_site_nei_C_nei_inv, C_site_nei_phi)) ;
     wa[i] = dot_product(C_site_nei_C_nei_inv , to_vector(append_row(w, wa))[nearind_sample[i,]]) + sqrt(wa_var[i]) * noise[n_point+i]; 
  }
  wA = log(A1 * exp(wa));
}

model{
  beta_point ~ normal(0, 5);
  beta_area ~ normal(0, 5);
  sigma_sq ~ inv_gamma(2, 1);
  tau_sq ~ inv_gamma(2, 1);
  phi ~ normal(300,100);
  noise ~ normal(0,1);
  Y ~ normal(X_point * beta_point + w, sqrt(tau_sq));
  Q ~ poisson_log(X_area * beta_area + wA);
}
